function [ea] = NSGA2(ea, varargin)
% Non-dominated Sorting Genetic Algorithm - II

global logger state

N = ea.param.pop_size;
assert(mod(N,4) == 0, 'Population size should be multiple of 4.');

write(logger, 'Starting NSGA-II\n');

% Generation 1
state.gen_id = 1;
pop = Population(ea.object, N, ea.prob);
pop = init(pop, varargin{:});
pop = eval(pop, ea.analysis);
pop = sort_nd_maxcv(pop);
ea = stats(ea, pop, 'ini');

% Generations 2-N
for i = 2:ea.param.generations
	state.gen_id = i;
	childpop = evolve(pop);
	childpop = eval(childpop, ea.analysis);
	pop = pop + childpop;
	pop = sort_nd_maxcv(pop);
	pop = reduce(pop, N);
	ea = stats(ea, pop, 'gen');

	if ea.param.max_fn_evals > 0 && pop.fn_evals >= ea.param.max_fn_evals
		break
	end
end

ea.pop = pop;
ea = stats(ea, pop,'fin');
return


%% NSGA-II evolution strategy
function [childpop] = evolve(pop)
N = pop.size;
childpop = Population(pop, N);
a1 = randperm(N);
a2 = randperm(N);

for i = 1:4:N
	p1 = tournament(pop, a1(i), a1(i+1));
	p2 = tournament(pop, a1(i+2), a1(i+3));
	[c1, c2] = crossover(pop, 'SBX', p1, p2);
	childpop.x{i} = c1;
	childpop.x{i+1} = c2;
	p1 = tournament(pop, a2(i), a2(i+1));
	p2 = tournament(pop, a2(i+2), a2(i+3));
	[c1, c2] = crossover(pop, 'SBX', p1, p2);
	childpop.x{i+2,:} = c1;
	childpop.x{i+3,:} = c2;
end

% Mutation
for i = 1:childpop.size
	childpop.x{i} = mutation(childpop, 'POLY', childpop.x{i});
end
	
return
